<?php

/*********************************************************************************

           TPLN  by HAYOUN Laurent

           website: http://tpln.sourceforge.net
           email: php_work@hotmail.com
           License : LGPL

**********************************************************************************/
/******************* Structure d'un fichier ***************************************


   f - f_no - name                 - string     // nom de fichier
            - buffer               - string     // contenu du fichier
            - items                - array      // contiens tous les items
            - php_items            - array      // contiens les items avec $
            - cmd_items            - array      // contiens tous les items includes
            - create_cached_file   - bool       // cre le fichier en cache ?
            - time_started         - long int   // dbut du chrono
            - cache_expire         - bool       // expiration du cache ?
            - execution_time       - long int   // temps d'execution
            - chrono_started       - long int   // chrono started

            Attention raccourci pour les vrifications et performances

            - shortcut_blocs       - array      //
               |_ all              - array      // contient le nom de tous les blocs
               |_ used             - array      // contient les blocs appels par l'utilsateur
               |_ name             - none       // contient le nom du bloc
                    |_ items       - array      // contient les items

            - def_blocs            - array      // contiens tous les blocs structurs
                                                 // definis par l'utisateur lors d'un appel'
                                                 // voir plus bas leurs structures
*/
/******************* Structure d'un bloc ***************************************

  def_blocs - name                  - string     // nom du bloc
              |_ structure          - string     // contenu du bloc
              |_ parsed             - array      // contiens les sessions de blocs
              |_ is_looped          - boolean    // loop ?
              |_ children           - array      // contiens les bloc enfants

*************************************************************************************/

//Initialisation de l'include_path
$old_path = ini_get('include_path');
$os = ((stristr(getenv('SERVER_SOFTWARE'), 'win') || stristr(getenv('SERVER_SOFTWARE'), 'microsoft'))? ';' : ':');
$path = $os.ini_get('include_path');
ini_set('include_path', $path);

//Inclusion du fichier de configuration et de Db
require_once('TPLN_Cfg.php');
require_once('TPLN_Debug.php');
require_once('TPLN_Error.php');
require_once('TPLN_DB_Xtra.php');
require_once('TPLN_DB.php');

// on remet l'include path au fichier appellant
ini_set('include_path', $old_path);
require_once('DB.php');   // inclusion de PEAR DB

/**
<docgeny>
<menu name="presentation">
 <name lang="fr">Prsentation de TPLN</name>
 <name lang="en">Presentation of TPLN</name>

 <description lang="fr">
 TPLN est un moteur de template en Php pour Php.
 Le code Php est spar du design ce qui permet un
 facilit de mise  jour du design de vos documents et
 de permettre galement de sparer les tches entre
 le dveloppeur Php et le Webdesigner.
 <br>
 <br>
 <img src="../img/explain.gif" border="0" align="center">
 </description>

 <description lang="en">
 TPLN is a template class written in Php for Php.
 The Php code separated of the design what
 facilitates the update of the design of your documents
 and to permit to separate work between
 developer Php and the Webdesigner.
 <br>
 <br>
 <img src="../img/explain.gif" border="0" align="center">
 </description>
</menu>

<menu name="template_file">
 <name lang="fr">Les fichiers Templates</name>
 <name lang="en">Template files</name>

 <description lang="fr">
 Les fichiers templates sont des fichiers qui comportent
 la mise en page de votre page.
 <br>
 <br>
 Ces fichiers peuvent ports n'importe quelles extensions  (tpl, txt, php, .. )
 </description>

 <description lang="en">
 Template files are files that have the webdesign of your generated page.
 These files can have any extensions (include php files)
 </description>
</menu>

<submenu name="declare_var" parent="template_file">
 <name lang="fr">Dclaration d'une variable</name>
 <name lang="en">Declaration of a variable</name>

 <description lang="fr">
 La dclaration des variables se fait  l'aide des accolades {} au sein de votre fichier template.
 </description>

 <description lang="en">
 La dclaration des variables se fait  l'aide des accolades {} au sein de votre fichier template.
 </description>

 <example>
 <html>
 <body>
{my_variable}
 </body>
 </html>
 </example>

 <warning lang="fr">
 Le nom des variables ne doit pas possder l'un de ces caractres """, "'", "*", " ", "\n","\t".
 Les variables Php peuvent tre incluses au sein de votre template comme ci {$variable}.
 </warning>

 <warning lang="en">
 The name of the variables must not possess one of these characters """, "'", "* ", " ", "\ n", "\ t."
 Php variable can be included in your template as here {$variable}.
 </warning>


</submenu>

<submenu name="declare_bloc" parent="template_file">
 <name lang="fr">Dclaration d'un bloc</name>
 <name lang="en">Declaration of a bloc</name>

 <description lang="fr">
 La dclaration des blocs se fait  l'aide des '&lt;bloc::nom>&lt;/bloc::nom&gt;' au sein de votre fichier template
 </description>

 <description lang="en">
 The declaration of the blocs makes itself with the help of the accolades '&lt;bloc::nom>&lt;/bloc::nom&gt;' in your file template.
 </description>

 <example>
 <html>
 <body>
 <bloc::my_bloc> {variable} </bloc::my_bloc>
 </body>
 </html>
 </example>

 <warning lang="fr">
 Le nom des blocs ne doit pas possder l'un de ces caractres """, "'", "*", " ", "\n","\t".
 </warning>

 <warning lang="en">
 The name of the blocs must not possess one of these characters """, "'", "* ", " ", "\ n", "\ t."
 </warning>


</submenu>

<submenu name="before_starting" parent="template_file">
 <name lang="fr">Avant de commencer</name>
 <name lang="en">Before beginning</name>

 <description lang="fr">
 Avant de commencer vous devez diter le fichier TPLN_Cfg.php dans le rpertoire TPLN.
 </description>

 <description lang="en">
 Before beginning, you need configure the present global variables in the TPLN_Cfg.php file within the TPLN directory.
 </description>
</submenu>

<menu name="functions">
 <name lang="fr">Les fonctions</name>
 <name lang="en">Functions</name>

 <description lang="fr">
 TPLN est une classe PHP. Elle ncessite cette dclaration.
 </description>

 <description lang="en">
 TPLN is a PHP class. It requires this declaration.
 </description>

 <example>
 <?php

 include("TPLN/TPLN.php"); // Inclusion of file
 $TPLN = &new TPLN; // Object declaration

 // ... suit of program

 ?>
 </example>

</menu>
</docgeny>
**/

class TPLN extends TPLN_DB
{
 var $TPLN_version = "1.6.5";
 var $def_tpl = array(); // liste des template dfini avec un nom
 var $f_no = -1; // indice de fichier
 var $f = array(); // c'est un array qui contient toutes les proprits
 var $chrono_type = array(); // on veut le chrono de qui ALL ou pas ?


// fonction constructrice de classe
function TPLN()
{ 
  // regarde si le repertoire par dfaut existe
  if(TPLN_DEFAULT_IND == 1)
   {
      if(!is_dir(TPLN_DEFAULT_PATH))
          {

            if(!@mkdir(TPLN_DEFAULT_PATH,0755))
             {
              $this->_Error(7.2);
              return;
             }

           clearstatcache();
          }
   } 
  
}




/********************************************
              General
*********************************************/

/**
<docgeny>
<submenu name="open" parent="functions">
 <name lang="fr">Open</name>
 <name lang="en">Open</name>

 <function>bool Open( [string filename], [ CACHED, [int time = not specified => default time] ] )</function>

 <description lang="fr">
 La fonction Open() sert  ouvrir le fichier template le fichier
 peut avoir n'importe quelle extension (.htm, .html, .tpl, .php ... ).
 <br>
 <br>
 Si aucun nom de fichier n'est defini, TPLN ouvriera le fichier ayant le meme nom que le script suivi de l'extension par defaut'
 </description>

 <description lang="en">
 The Open() function serves to open
 the file template the file can have any extension (. htm. html. tpl,php ...).
 <br>
 <br>
 If filename is empty, TPLN will open the template which has the same name as your script with your default extension
 </description>

 <example>
 <?php

 // Simple opening
 $TPLN->Open("my_file.html");

 // or you can do
 $TPLN->Open(); // will open the same name as your script + default extension

 ?>

 <?php

 // Opening with cache to 3600 seconds is 1 hour :
 // if not specified TPLN use the default configuration in TPLN_Cfg.php

 if ( !$TPLN ->Open("my_file",CACHED, 3600) )
 {
  //code to execute to generate the file
 }
 $TPLN->Write(); // print result

 ?>
 </example>

 <warning lang="fr">
 Si l'extension du fichier est php, php3, phtml, php4
 alors le fichier sera execut automatiquement par le parseur php.
 </warning>

 <warning lang="en">
 If the extension of the file is php, php3, phtml, php4,
 then the file will be executed automatically by Php.
 </warning>

 <see>DefineTemplate</see>
</submenu>
</docgeny>
**/

function Open($file="",$cached='',$cached_time='')
 {


  $this->f_no++;  // incremante

/** initialisation des types de variables pour EALL ***************************/
  $this->f["$this->f_no"]["name"] = NULL;
  $this->f["$this->f_no"]["buffer"] = NULL;
  $this->f["$this->f_no"]["items"] = array();
  $this->f["$this->f_no"]["php_items"] = array();
  $this->f["$this->f_no"]["cmd_items"] = array();

  $this->f["$this->f_no"]["create_cached_file"] = 0;
  $this->f["$this->f_no"]["time_started"] = 0;
  $this->f["$this->f_no"]["cache_expire"] = 0;
  $this->f["$this->f_no"]["execution_time"] = 0;
  $this->f["$this->f_no"]["chrono_started"] = 0;

  $this->f["$this->f_no"]["shortcut_blocs"] = array();
  $this->f["$this->f_no"]["shortcut_blocs"]["all"] = array();
  $this->f["$this->f_no"]["shortcut_blocs"]["used"] = array();

  $this->f["$this->f_no"]["shortcut_blocs"]["name"] = NULL;
  $this->f["$this->f_no"]["shortcut_blocs"]["name"]["items"] = array();

  $this->f["$this->f_no"]["def_blocs"] = array();

  $this->chrono_type["$this->f_no"] = NULL;
/******************************************************************************/

  // ajout du nom de fichiers
  if(empty($file))
  {
     $file = basename($_SERVER['PHP_SELF']);
     // on prend l'extension et on l'efface
     preg_match("/\.([^\.]*$)/", $file, $elts);

     if(count($elts) > 1)
       $file = str_replace($elts[count($elts)-1],TPLN_DEFAULT_EXT,$file);
  }


  // verifie l'existence de l'extension et du rpertoire par dfault
  if(TPLN_DEFAULT_IND == 1)
  {
     if(!preg_match("/\./",basename($file)))
       {$file .= ".".TPLN_DEFAULT_EXT;}// met l'extension par defaut s'il ni en a pas

     $file = TPLN_DEFAULT_PATH."$file";// met le repertoire par defaut
  }

  $this->f["$this->f_no"]["name"] = $file;

  $time = explode(' ',microtime());
  $this->f["$this->f_no"]["time_started"] = time(); // dans pour le cache  la seconde
  $this->f["$this->f_no"]["chrono_started"] = $time[1] + $time[0];

  // on dfinit le chrono
  if(!defined('TPLN_CHRONO_STARTED'))
   {
    $chrono = $time[1] + $time[0];
    define('TPLN_CHRONO_STARTED',$chrono);
   }

   // on regarde si le fichier est en cache
   if(!empty($cached))
    {
      $this->_CacheDirExists();

      if(empty($cached_time)){$cached_time = TPLN_CACHE_TIME;}
      $this->f["$this->f_no"]["cache_expire"] = $this->f["$this->f_no"]["time_started"]+$cached_time; // cache d'expiration

      // on regarde le fichier est encore en cache
      // fichier existe && sa date de creation est <= au temps de cache
      if(file_exists(TPLN_CACHE_DIR.$this->f["$this->f_no"]["name"]) && $this->_InCachePeriod())
       {
        $this->_GetCachedFile();
        return true;
       }
      else
       {
        $this->f["$this->f_no"]["create_cached_file"] = 1;

       }
    }

     $this->f["$this->f_no"]["buffer"] = $this->GetFile($this->f["$this->f_no"]["name"]);

     // remplace les variables avec $
     if(TPLN_PARSE_GLOBALS == 1)
      {
        $this->ParseGlobals();
      }

     // on remplace les fichiers  inclure
     $this->_CaptureIncludeCmd(); // capture des commandes include
     // parsing et valuation si necessaire
     if(count($this->f["$this->f_no"]["cmd_items"]) > 0)
     {
        $this->_ParseAllIncludeCmd();
        $this->f["$this->f_no"]["cmd_items"] = array(); // on efface
     }

     // on reremplace les $ si contenues dans fichiers inclu
     if(TPLN_PARSE_GLOBALS == 1)
      {
        $this->ParseGlobals();
      }


     $this->f["$this->f_no"]["shortcut_blocs"]["all"] = $this->_CaptureAllBlocs();
     $this->_EndBlocVerify();
     $this->_DualBlocVerify();

     // Capture des items apres verification des blocs
     $this->f["$this->f_no"]["items"] = $this->_CaptureItems();
     $this->_CaptureItemsInEachBloc();

   // debugger
   $this->_StructOpen();

   if($this->f["$this->f_no"]["create_cached_file"] == 1){ return false;}
 }
 
/**
<docgeny>
<submenu name="directiowrite" parent="functions">
 <name lang="fr">DirectIOWrite</name>
 <name lang="en">DirectIOWrite</name>

 <function>void DirectIOWrite( [string filename])</function>

 <description lang="fr">
 La fonction DirectIOWrite() est une macro qui permet d'ouvrir et d'ecrire un template.
 </description>

 <description lang="en">
 DirectIOWrite() function is a macro that permits to open and write a template.
 </description>

 <example>
 <?php

 $TPLN->DirectIOWrite("my_template.html");
 
 // is the same as
 $TPLN->Open("my_template.html");
 $TPLN->Write();

 ?>
 </example>

 <see>Open Write</see>
</submenu>
</docgeny>
**/
function DirectIOWrite($file="")
{
         $c_id = $this->f_no; // on sauve le numero actuel 
         
         $this->Open($file);
         $this->Write();
         
         $this->f_no = $c_id;// on remet le numero           
}

/**
<docgeny>
<submenu name="directiooutput" parent="functions">
 <name lang="fr">DirectIOOutput</name>
 <name lang="en">DirectIOOutput</name>

 <function>string DirectIOOutput( [string filename])</function>

 <description lang="fr">
 La fonction DirectIOOutput() est une macro qui permet d'ouvrir et de retourner un template.
 </description>

 <description lang="en">
 DirectIOOutput() function is a macro that permits to open and return a template.
 </description>

 <example>
 <?php

 $TPLN->DirectIOOutput("my_template.html");
 
 // is the same as
 $TPLN->Open("my_template.html");
 $file = $TPLN->Output();
 echo "$file";

 ?>
 </example>

 <see>Open Write OutPut</see>
</submenu>
</docgeny>
**/
function DirectIOOutput($file="")
{
         $c_id = $this->f_no; // on sauve le numero actuel
         
         $this->Open($file);  
         $output = $this->OutPut(); 
         
         $this->f_no = $c_id;// on remet le numero 
         
         return $output;
           
                 
}

/**
<docgeny>
<submenu name="directiosave" parent="functions">
 <name lang="fr">DirectIOSave</name>
 <name lang="en">DirectIOSave</name>

 <function>string DirectIOOutput( [string filename],string path)</function>

 <description lang="fr">
 La fonction DirectIOSave() est une macro qui permet d'ouvrir et de sauver un template.
 </description>

 <description lang="en">
 DirectIOSave() function is a macro that permits to open and save a template.
 </description>

 <example>
 <?php

 $TPLN->DirectIOSave("my_template.html","files/myne.html");
 
 // is the same as
 $TPLN->Open("my_template.html");
 $TPLN->SaveTemplate("files/myne.html");

 ?>
 </example>

 <see>Open SaveTemplate</see>
</submenu>
</docgeny>
**/
function DirectIOSave($file="",$path)
{
         $c_id = $this->f_no; // on sauve le numero actuel               
         
         $this->Open($file);
         $this->SaveTemplate($path); 
         
          $this->f_no = $c_id;// on remet le numero           
                   
}


/**
<docgeny>
<submenu name="definetemplate" parent="functions">
 <name lang="fr">DefineTemplate</name>
 <name lang="en">DefineTemplate</name>

 <function>void DefineTemplate( array filename )</function>

 <description lang="fr">
 La fonction DefineTemplate() sert  ouvrir plusieurs templates en leurs donnant un nom.
 </description>

 <description lang="en">
 DefineTemplate() function serves to open multiple template using a name.
 </description>

 <example>
 <?php

 $TPLN->DefineTemplate(array("TOP"=>"top.html","BOTTOM" => "bottom.html");

 ?>
 </example>

 <see>SetTemplate</see>
</submenu>
</docgeny>
**/
function DefineTemplate($arr)
{
  if(!is_array($arr))
   {
     $this->_Error(12);
     return;
   }

   foreach($arr as $key=>$val)
     {
       $this->Open($val);
       $this->def_tpl[count($this->f)-1] = $key;
     }
}


/**
<docgeny>
<submenu name="settemplate" parent="functions">
 <name lang="fr">SetTemplate</name>
 <name lang="en">SetTemplate</name>

 <function>void SetTemplate( string|int file )</function>

 <description lang="fr">
 La fonction SetTemplate() sert  slectionner un template ouvert
 par la fonction <a href="Open.html">Open()</a> ou
 <A href="DefineTemplate.html">DefineTemplate()</A> soit par un nom ou
 par un numro.
 </description>

 <description lang="en">
 SetTemplate() function serves
 to select a template opened with <a href="Open.html">Open()</a> or
 <A href="DefineTemplate.html">DefineTemplate()</A> by his name or his number.
 </description>

 <example>
 <?php
 // SetTemplate with Definetemplate

 $TPLN->DefineTemplate(array("TOP"=>"top.html","BOTTOM" => "bottom.html");
 $TPLN->SetTemplate("TOP");//choose template TOP
 ... //code to execute

 // SetTemplate with Open :

 $TPLN->Open("top.html");//number 0
 $TPLN->Open("bottom.html"); //number 1

 $TPLN->SetTemplate(0);//we choose template TOP
 ... //code to execute
 ?>
 </example>

 <warning lang="fr">
 Depuis TPLN 1.4, cette function est obselte utilis plutt <a href="ChangeTemplate.html">ChangeTemplate()</a>
 </warning>

 <warning lang="en">
 Since TPLN 1.4, this function is not supported used <a href="ChangeTemplate.html">ChangeTemplate()</a> instead.
 </warning>


 <see>Open DefineTemplate ChangeTemplate</see>
</submenu>
</docgeny>
**/

function SetTemplate($key)
{
  $this->ChangeTemplate($key);
}
/**
<docgeny>
<submenu name="changetemplate" parent="functions">
 <name lang="fr">ChangeTemplate</name>
 <name lang="en">ChangeTemplate</name>

 <function>void ChangeTemplate( string|int file )</function>

 <description lang="fr">
 La fonction ChangeTemplate() sert  slectionner un template ouvert
 par la fonction <a href="Open.html">Open()</a> ou
 <A href="DefineTemplate.html">DefineTemplate()</A> soit par un nom ou
 par un numro.
 </description>

 <description lang="en">
 ChangeTemplate() function serves
 to select a template opened with <a href="Open.html">Open()</a> or
 <A href="DefineTemplate.html">DefineTemplate()</A> by his name or his number.
 </description>

 <example>
 <?php

 // ChangeTemplate with Definetemplate

 $TPLN->DefineTemplate(array("TOP"=>"top.html","BOTTOM" => "bottom.html");
 $TPLN->ChangeTemplate("TOP");//choose template TOP
 ... //code to execute

 // ChangeTemplate with Open :

 $TPLN->Open("top.html");//number 0
 $TPLN->Open("bottom.html"); //number 1

 $TPLN->ChangeTemplate(0);//we choose template TOP
 ... //code to execute
 ?>
 </example>

 <see>Open DefineTemplate</see>
</submenu>
</docgeny>
**/

function ChangeTemplate($key)
{
  if(is_string($key))
  {
    // on prend l'id
     if(!in_array($key,$this->def_tpl,TRUE))
     {
      $this->_Error(11,$key);
      return;
     }

    $key = array_search($key,$this->def_tpl,TRUE);

  }

  if(is_int($key))
  {
    if($key < 0 || $key >= count($this->f))
      {
       $this->_Error(10,$key);
      }
   }

  $this->f_no = $key;
}



/**
<docgeny>
<submenu name="parseglobals" parent="functions">
 <name lang="fr">ParseGlobals</name>
 <name lang="en">ParseGlobals</name>

 <function>void ParseGlobals()</function>

 <description lang="fr">
 La fonction ParseGlobals() sert  parser une variable Php  une variable
 contenue dans le template.
 </description>

 <description lang="en">
 The function ParseGlobals() serves to parse a Php variable to a variable
 contained inside the template.
 </description>

 <warning lang="fr">
 Cette fonction est automatiquement appele par dfaut  l'ouverture du template.
 </warning>

 <warning lang="en">
 This function is called automatically by default at the template openning.
 </warning>

</submenu>
</docgeny>
**/

function ParseGlobals()
{
  $this->f["$this->f_no"]["php_items"] = $this->_CaptureItems("","PHP");

  if(count($this->f["$this->f_no"]["php_items"]) == 0)
  {
   return;
  }

  foreach($this->f["$this->f_no"]["php_items"] as $item)
   {
     // on regarde si la variable est _GET ou _POST ou _SESSION
     if(substr($item,0,1) != '_') // pas de signe $_ au dbut
      {
        if(!isset($GLOBALS["$item"]))
        {
          $GLOBALS["$item"] = NULL;
        }

        $replace = $GLOBALS["$item"];
      }
      else
      {
       $replace = '$'.$item;
       eval("\$replace = $replace;");
      }

     $item = '$'.$item;
     $item = str_replace('$','\$',$item);
     $item = str_replace('[','\[',$item);
     $item = str_replace(']','\]',$item);
     $this->f["$this->f_no"]["buffer"] = $this->_ReplaceItem($item, $replace, $this->f["$this->f_no"]["buffer"]);
   }
}


/**
<docgeny>
<submenu name="parse" parent="functions">
 <name lang="fr">Parse</name>
 <name lang="en">Parse</name>

 <function>void Parse( string item , string replace [, string functions ] )</function>

 <description lang="fr">
 La fonction Parse() sert  remplacer une variable dfinie au sein de votre fichier template.
 TPLN utilise un pseudo langage objet de style javascript pour accder  votre variable grce au sparateur ".".
 <br>
 <br>
 Cette fonction possde galement un paramtre optionnel functions.
 Celui ci sert  formater le texte de replace.
 Ces options sont les suivantes :
 <br>
 <ul>
 <li><b>|B</b> ajoute les balises <b></b>  replace ce qui le rend gras</li>
 <li><b>|I</b> ajoute les balises <i></i>  replace ce qui le rend italique</li>
 <li><b>|U</b> ajoute les balises <u></u>  replace ce qui le rend soulign</li>
 <li><b>|nom</b> de la fonction Php n'importe quelle fonction Php (<a href="http://www.php.net/manual/fr/function.ucfirst.php" target="_blank">ucfirst</a> par exemple)</li>
 </ul>
 </description>

 <description lang="en">
 The Parse() function serves to replace a variable defined in your file template.
 TPLN uses a pseudo language object of style javascript to reach your variable thanks to the separator ". ".
 <br>
 <br>
 This function also possesses a parameter optional functions.
 The one serves to format the text here of replace.
 These options are the following :
 <br>
 <ul>
 <li><b>|B</b>  adds <b></b> to replace what makes it bold
 <li><b>|I</b> adds <i></i> to replace what makes it italic
 <li><b>|U</b> adds<u></u> to replace what makes it underline
 <li><b>|Php</b> function name any Php functions (<a href="http://www.php.net/manual/en/function.ucfirst.php" target="_blank">ucfirst</a> for example)
 </description>

 <example>
 <?php

 // Parse
 $TPLN->Parse("my_variable","replace");

 // Parse in bloc
 $TPLN->Parse("my_bloc.my_variable","replace");

 // Parse in bloc with format
 $TPLN->Parse("my_bloc.my_variable","replace","|B");

 ?>
 </example>
</submenu>
</docgeny>
**/
function Parse($path,$replace,$functions="")
 {
  // fonction de formatage des donnes
  $replace = $this->_ApplySpecialFunction($replace,$functions);

  if($this->_IsBlocDesired($path))  // est ce un bloc ?
   {
      $item = $this->_GetItem($path);// on prend l'item
      $fathers_arr = $this->_GetFathers($path,'ARRAY'); // les pres ds un array
      $bloc = $this->_GetFather($path); // on prend le pre

      if(!$this->BlocExists($bloc)){
          $this->_Error(2,$this->f["$this->f_no"]["name"],$bloc);}

      $this->_ItemVerify($item,$bloc);// verification


      if(!$this->_IsDefined($path)) // le chemin est il enregistr
        {
           $this->_DefineBloc($path); // defini les blocs un par un
        }

      $this->_DirectParseInBloc($fathers_arr,$item,$replace);
   }
  else
  {
     $this->_ItemVerify($path);// verification
     $this->_DirectParseInFile($path, $replace); // parsing direct dans le fichier
  }

}

function _DirectParseInFile($item, $replace)
{
  $this->f["$this->f_no"]["buffer"] = $this->_ReplaceItem($item,$replace,$this->f["$this->f_no"]["buffer"]);
}

function _DirectParseInBloc($fathers_arr,$item,$replace)
{
  $b_ref = &$this->f["$this->f_no"]["def_blocs"];

  // on atteint le bloc
  for($i=0;$i<count($fathers_arr);$i++){
      $b_ref = &$b_ref["{$fathers_arr[$i]}"];

      if($i < count($fathers_arr)-1){
          $b_ref = &$b_ref["children"];}}

  // on regarde si il y eu un loop > 0
  $loop_nb = count($b_ref["parsed"]);
  $b_ref["parsed"][$loop_nb-1] = $this->_ReplaceItem($item,$replace,$b_ref["parsed"][$loop_nb-1]);
}



/**
<docgeny>
<submenu name="fastparse" parent="functions">
 <name lang="fr">FastParse</name>
 <name lang="en">FastParse</name>

 <function>void FastParse( string item [, string functions ] )</function>

 <description lang="fr">
 La fonction FastParse() est similaire  la fonction <a href="parse.html">Parse()</a> except quelle vite de rcrire les variables dfinies
 </description>

 <description lang="en">
 FastParse() function is the same function as <a href="parse.html">Parse()</a> except it evits you to rewrite a variable php
 </description>

 <example>
 <?php

 // FastParse
 $name= "my_name";
 $TPLN->FastParse("name");

 // the same is
 $TPLN->Parse("name",$name);
 ?>
 </example>

 <see>Parse</see>
</submenu>
</docgeny>
**/

function FastParse($path,$functions="")
{

  // Est ce un bloc alors on remplace
  if($this->_IsBlocDesired($path))
   {
      $item =  $this->_GetItem($path);// on prend le dernier item
   }
  else
   {
      $item = $path;
   }

  $this->Parse($path, $GLOBALS[$item], $functions);
}


/*
<docgeny>
<submenu name="parsedbrow" parent="db">
 <name lang="fr">ParseDBRow</name>
 <name lang="en">ParseDBRow</name>

 <function>void ParseDBRow(string|array blocname,[string parse_function])</function>

 <description lang="fr">
 La fonction ParseDBRow() sert  parser toutes les variables d'un bloc par les valeurs retournes par row de votre requte.

 Le paramtre parse_function est celui des fonctions de Parse(), vous pouvez mettre des
 fonctions php  l'intrieur.
 </description>

 <description lang="en">
 ParseDBRow() serves to parse all variables of a bloc by values returned by a row of your query.

 Parse_function is the same as Parse() function, you can put php function.
 </description>

 <example>
 <?php
 $TPLN->Open("template.html");

 $TPLN->DbConnect();
 $TPLN->DoQuery("SELECT * FROM my_table");
 $TPLN->ParseDbRow("bloc1","|strtoupper"); // Parse field and use php strtoupper function
 $TPLN->DbClose();

 $TPLN->Write();
 ?>
 </example>

</submenu>
</docgeny>
*/
function ParseDBRow($bloc,$func="")
{
  $this->_PathVerify($bloc);

  while($row = $this->DBFetchAssoc())
  {
    $keys = @array_keys($row);// prise du mon des clefs

    foreach($keys as $key)
     {
       if($this->ItemExists($key,$bloc))
        {
          if(!empty($func))
              $this->Parse("$bloc.$key",$row["$key"],$func);
          else
              $this->Parse("$bloc.$key",$row["$key"]);
        }
     }
     $this->Loop("$bloc");
  }
}

/*
<docgeny>
<submenu name="parsedbfield" parent="db">
 <name lang="fr">ParseDBField</name>
 <name lang="en">ParseDBField</name>

 <function>void ParseDBField(string $blocname, array fields, [string parse_function])</function>

 <description lang="fr">
 La fonction ParseDBField() sert  parser toutes les variables d'un bloc par les valeurs retournes par field de votre requte.
 </description>

 <description lang="en">
 ParseDBField() serves to parse all variables of a bloc by values returned by a field_name of your query.
 </description>

 <example>
 <?php
 $TPLN->Open("template.html");

 $TPLN->DbConnect();
 $fields = $TPLN->GetFields("SELECT * FROM my_table");
 $TPLN->ParseDBField("bloc1",$fields,"|strtoupper"); // Parse field and use php strtoupper function
 $TPLN->DbClose();

 $TPLN->Write();
 ?>
 </example>

 <warning lang="fr">
 Depuis TPLN 1.5
 </warning>

 <warning lang="en">
 Since TPLN 1.5
 </warning>

 <see>Parse ShowRecords GetFields ParseDBRow</see>

</submenu>
</docgeny>
*/
function ParseDBField($bloc,$fields, $func="")
{
  /*echo "<pre>";
  print_r($this->f);
  echo "</pre>"; */

  // verification du chemin
  $this->_PathVerify($bloc);

  // on simule que l'on prend un item
  $last_bloc = $this->_GetItem($bloc);

  // control  faire sur le chemin !
  for($i=0;$i < count($fields); $i++)
  {
     $field_name = $fields[$i];

      if($this->ItemExists($field_name,$last_bloc))
      {
         if(!empty($func))
              $this->Parse("$bloc.$field_name",$field_name,$func);
          else
              $this->Parse("$bloc.$field_name",$field_name);
      }
  }
}


/**
<docgeny>
<submenu name="loadarrayinbloc" parent="functions">
 <name lang="fr">LoadArrayInBloc</name>
 <name lang="en">LoadArrayInBloc</name>

 <function>void LoadArrayInBloc( array array)</function>

 <description lang="fr">
 La fonction LoadArrayInBloc() sert  remplacer dans un bloc entier les items par les pairs clefs d'un array.
 </description>

 <description lang="en">
 LoadArrayInBloc() serves to parse an entire bloc by the associated key of an array.
 </description>

 <example>
 <?php

 $array[] = array("User"=>"Me", "Password"=>"test");
 $TPLN->LoadArrayInBloc("my_bloc" , $arr);

 ?>
 </example>

</submenu>
</docgeny>
**/
function LoadArrayInBloc($path,$arr)
{
  if(count($arr) == 0){return;}

  foreach($arr as $current_arr)
   {
      $keys = @array_keys($current_arr);// prise des clefs

      foreach($keys as $key)
       {
         $this->Parse("$path.$key", $current_arr[$key]);
       }

       $this->Loop($path);
   }

}

/**
<docgeny>
<submenu name="loop" parent="functions">
 <name lang="fr">Loop</name>
 <name lang="en">Loop</name>

 <function>void Loop( string path )</function>

 <description lang="fr">
 La fonction Loop() est applique  un bloc pour signaler sa rptition.
 </description>

 <description lang="en">
 The Loop() function is applied to a bloc to signal its repetition.
 </description>

 <example>
 <?php

 for($i=0, $i < 10; $i++){
 $TPLN->Parse("my_bloc.my_variable","$i");
 $TPLN->Loop("my_bloc");

 ?>
 </example>

</submenu>
</docgeny>
**/

function Loop($path)
{

  // vrification de l'existence du chemin comme defini
  if(!$this->_IsDefined($path,'NOITEM'))
  {
      $this->_Error(4.1,$this->f["$this->f_no"]["name"],$path);
      return;
  }

  $fathers_arr = $this->_GetFathers($path,'ARRAY',0);

  // touche le bloc
  $b_ref = &$this->f["$this->f_no"]["def_blocs"];

 for($i=0;$i<count($fathers_arr);$i++)
 {
   $b_ref = &$b_ref[$fathers_arr[$i]];

   if($i < count($fathers_arr)-1)
    {
       $b_ref = &$b_ref["children"];
    }
 }

 // toute la gnration qui suit est remplac donc
 if(count($b_ref["children"]) > 0)
  {
      //$level = count($fathers_arr)-1;
      $child_blocs = $this->_GetNextGenerationBlocs($fathers_arr);

      // on encapsule les blocs
      $this->_EncapsuleBlocs($fathers_arr,$child_blocs);
  }

 // on ajoute une session parse
 // on incrmente la valeur de loop
 $b_ref["parsed"][] = $b_ref["structure"];
 $b_ref["is_looped"] = 1;
}



function _GetNextGenerationBlocs($bloc_arr)
{
  // touche le bloc
  $b_ref = &$this->f["$this->f_no"]["def_blocs"];

 for($i=0;$i<count($bloc_arr);$i++)
 {
   $b_ref = &$b_ref[$bloc_arr[$i]];

   if($i < count($bloc_arr)-1)
    {
       $b_ref = &$b_ref["children"];
    }
 }

 $b_names = array_keys($b_ref["children"]);


  return $b_names;
}




function _EncapsuleBlocs($bloc_arr, $child_blocs)
{
  // on encapsule chacun des enfants CAD
  // rtrcissement du bloc s'il a des enfants
  // parsing chez le pre du fils parsed
  // remize  zero du bloc fils
  foreach($child_blocs as $children)
  {
     // touche le bloc
     $b_ref = &$this->f["$this->f_no"]["def_blocs"];

     for($i=0;$i<count($bloc_arr);$i++)
      {
       $b_ref = &$b_ref[$bloc_arr[$i]];

       if($i == count($bloc_arr)-1)
        {
           $father_parsed = &$b_ref["parsed"][count($b_ref["parsed"])-1]; // c'est le dernier
        }

       $b_ref = &$b_ref["children"];
      }

    $children_all_parsed = &$b_ref["$children"]["parsed"];
    $children_structure = &$b_ref["$children"]["structure"];
    $children_parsed = "";

  // bloc
   if(count($children_all_parsed) == 1)
    {
       $children_parsed = $children_all_parsed['0'];
    }

   if(count($children_all_parsed) > 1)
   {
         for($l=0; $l<count($children_all_parsed)-1; $l++)
         {
             $children_parsed =  $children_parsed.$children_all_parsed[$l];
         }
      }


    // parsing chez le pre
    $father_parsed = $this->_ReplaceBloc($children,$children_parsed,$father_parsed);

    // remize  zero du bloc child
    $children_all_parsed = array($children_structure);
    unset($children_parsed);
  }
}




/**
<docgeny>
<submenu name="eraseitem" parent="functions">
 <name lang="fr">EraseItem</name>
 <name lang="en">EraseItem</name>

 <function>void EraseItem( string item )</function>

 <description lang="fr">
 La fonction EraseItem() sert  effacer une variable.
 </description>

 <description lang="en">
 The EraseItem() function serves to erase a variable.
 </description>

 <example>
 <?php
 $TPLN->EraseItem("my_variable");

 // in a bloc:
 $TPLN->EraseItem("my_bloc.my_variable");
 ?>
 </example>

</submenu>
</docgeny>
**/

function EraseItem($item)
{
 $this->Parse($item,"");
}


/**
<docgeny>
<submenu name="erasebloc" parent="functions">
 <name lang="fr">EraseBloc</name>
 <name lang="en">EraseBloc</name>

 <function>void EraseBloc( string blocname )</function>

 <description lang="fr">
 La fonction EraseBloc() sert  effacer un bloc.
 Si le bloc est unique alors le remplacement se fait directement dans le template.
 </description>

 <description lang="en">
 The EraseBloc() function serves to erase a bloc.
 If bloc is unic the pazrser will delete bloc directly in template.
 </description>

 <example>
 <?php

 ....

 for($i=0, $i < 10; $i++){
 $TPLN->Parse("my_bloc.my_variable","$i");
 $TPLN->Loop("my_bloc");

 $TPLN->EraseBloc("my_bloc");
 ?>
 </example>

 <see>Loop</see>

</submenu>
</docgeny>
**/

function EraseBloc($path)
{
  $this->ParseBloc($path,"");

  /*$bloc_arr = $this->_GetFathers($path,'ARRAY',0);
  //  un niveau
  if(empty($type) && count($bloc_arr) == 1)
  {
     $this->ParseBloc($path,"");
     return;
  }

 // pas dans un loop
 // parsing simple chez un pere
 // on regarde si le pre existe d'abord
 if(empty($type))
 {
   $n_bloc = array_slice($bloc_arr, 0,count($bloc_arr)-1);
   $n_path = join('.',$n_bloc);


   // regarde si le bloc est dfini ?
   if(!$this->_IsDefined($n_path,"NOITEM"))
   {
      $this->_DefineBloc($n_path.".none");
   }

   // on prend le pre parsed !
   $b_ref = &$this->f["$this->f_no"]["def_blocs"];

   for($i=0; $i<count($bloc_arr)-2; $i++)
   {
    $b_ref = &$b_ref[$n_bloc[$i]]["children"];
   }

   $father_name = $bloc_arr[count($bloc_arr)-2];
   $last_bloc = $bloc_arr[count($bloc_arr)-1];
   $all_parsed = &$b_ref[$father_name]["parsed"];
   $father_parsed = &$all_parsed[count($all_parsed)-1];

   /*echo "path $path \n";
   echo "last_bloc $last_bloc \n";
   echo "all parsed ";
   var_dump($all_parsed);
   echo "father_name $father_name \n";
   echo "father_parsed $father_parsed \n";

   $father_parsed = $this->_ReplaceBloc($last_bloc,"",$father_parsed);
   /*echo "new father_parsed $father_parsed \n";
   exit;

 }


if($type == "LOOP")
 {

    if(!$this->_IsDefined($path,"NOITEM"))
    {
      $this->_DefineBloc($path.".none");
    }

    $b_ref = &$this->f["$this->f_no"]["def_blocs"];

    for($i=0;$i<count($bloc_arr);$i++)
    {
      $b_ref = &$b_ref[$bloc_arr[$i]];

      if($i == count($bloc_arr)-2)
      {
        $father_parsed = &$b_ref["parsed"][count($b_ref["parsed"])-1];
        //echo "father_parsed $father_parsed";
        //echo "<hr>";
      }

      if($i == count($bloc_arr)-1)
      {
          $children_name = $bloc_arr[$i];
          //echo "children_name $children_name";
          //echo "<hr>";
      }

      if($i < count($bloc_arr)-1)
      {
            $b_ref = &$b_ref["children"];
      }
    }

    // le bloc n'existe pas
    if(empty($b_ref))
    {
    $this->_Error(5,$this->f["$this->f_no"]["name"],$path);
    return;
    }

    // on efface le bloc chez son pre
    if(count($bloc_arr) > 1)
    {
      $father_parsed = $this->_ReplaceBloc($children_name,"",$father_parsed);
    }

    $b_ref["parsed"] = array();// on l'efface sa session  zero
 }
*/

}


/**
<docgeny>
<submenu name="getfile" parent="functions">
 <name lang="fr">GetFile</name>
 <name lang="en">GetFile</name>

 <function>string GetFile( string filename [, string blocname ] )</function>

 <description lang="fr">
 La fonction GetFile() sert  retourner le contenu d'un fichier.
 Si le paramtre optionnel blocname est prcis alors la fonction retourne le contenu du bloc dfini dans le fichier.
 </description>

 <description lang="en">
 The GetFile() function serves to return the file content.
 If optionnal parameter blocname is precised the function will return blocname in the file.
 </description>

 <example>
 <?php
 // Getfile (open file file.html et return bloc1 of file.html)
 $file= $TPLN->GetFile("file.html","bloc1");
 ?>
 </example>

</submenu>
</docgeny>
**/

function GetFile($filename,$blocname="")
{

  if(!$fp = @fopen($filename, "r")) // ouverture du fichier en lecture
     {
      $this->_Error(0,$filename);
     }

      $filebuffer = @fread($fp,filesize($filename));
      @fclose($fp);
      clearstatcache(); // efface la mmoire tampon

      if(!empty($blocname)) // on veut capturee un bloc
        {
         $filebuffer = $this->_CaptureBloc($blocname,$filebuffer);
        }

 return $filebuffer;
}


/**
<docgeny>
<submenu name="parsebloc" parent="functions">
 <name lang="fr">ParseBloc</name>
 <name lang="en">ParseBloc</name>

 <function>void ParseBloc( string filename ,  string replace )</function>

 <description lang="fr">
 La fonction ParseBloc() sert  remplacer un bloc entier avec ses balises incluses.
 </description>

 <description lang="en">
 The ParseBloc() function serves to replace a whole block with its included beacons.
 </description>

 <example>
 <?php
 ...
 $TPLN->ParseBloc("my_bloc" , "bloc my_bloc is empty !");

 ?>
 </example>

</submenu>
</docgeny>
**/
function ParseBloc($path,$replace)
{
  $this->_PathVerify($path); // Verification de l'existence des blocs du chemin

  // definition du chemin
  if(!$this->_Isdefined($path,'NOITEM')) $this->_DefineBloc($path.".none");

  // bloc imbriqu ?
  if(!$this->_IsBlocDesired($path))
  {
     $this->f["$this->f_no"]["buffer"] = $this->_ReplaceBloc($path,$replace,$this->f["$this->f_no"]["buffer"]);
     $bloc_arr[] = $path;
  }
  else
  {
      // on doit aller dans le bloc pour l'liminer !
      $bloc_arr = $this->_GetFathers($path, "ARRAY",0);
  }

  // on efface le bloc
  $b_ref = &$this->f["$this->f_no"]["def_blocs"];

  // un seul bloc ?
  if(count($bloc_arr) == 1)
  {
     $b_ref = &$b_ref[$bloc_arr[0]];
  }
  else
  {
      // on efface dans le dernier pere le bloc en cours
      for($i=0;$i<count($bloc_arr);$i++)
      {
         $b_ref = &$b_ref[$bloc_arr[$i]];

         if($i == count($bloc_arr)-2)
         {
           $last_father_parsed = &$b_ref["parsed"][count($b_ref["parsed"])-1];
         }

         if($i < count($bloc_arr)-1)
              $b_ref = &$b_ref["children"];

      }

       $last_father_parsed = $this->_ReplaceBloc($bloc_arr[count($bloc_arr)-1],$replace,$last_father_parsed);

  }


  $b_ref["parsed"] = array(); // on efface sa session du dernier bloc !



  /*// est ce un bloc imbriqu
  if($this->_IsBlocDesired($path))
  {
    $last_bloc = $this->_GetItem($path);// on prend le dernier dans comme un item
  }

  if($this->_IsDefined($path,"NOITEM"))
     echo "$path defini !<br>";
  else
      echo "$path non dfini !<br>";



  if(!$this->BlocExists($path))
  {
     $this->_Error(2,$this->f["$this->f_no"]["name"],$path);
     return;
  }


  $this->f["$this->f_no"]["buffer"] = $this->_ReplaceBloc($path,$replace,$this->f["$this->f_no"]["buffer"]);
  */
}


/**
<docgeny>
<submenu name="reloadblocvars" parent="functions">
 <name lang="fr">ReloadBlocVars</name>
 <name lang="en">ReloadBlocVars</name>

 <function>void ReloadBlocVars(string path)</function>

 <description lang="fr">
 La fonction ReloadBlocVars() sert  recharger les variables d'un bloc
 contenue dans le template.
 </description>

 <description lang="en">
 The function ReloadBlocVars() serves to reload variable of a bloc
 contained inside the template.
 </description>

 <warning lang="fr">
 TPLN 1.5
 </warning>

 <warning lang="en">
 TPLN 1.5
 </warning>


</submenu>
</docgeny>
**/
function ReloadBlocVars($path)
{
 $this->_PathVerify($path);

 $last_bloc = $this->_GetItem($path);
 $this->f["$this->f_no"]["shortcut_blocs"][$last_bloc]["items"] = $this->_CaptureItems($last_bloc);

}


/**
<docgeny>
<submenu name="includefile" parent="functions">
 <name lang="fr">IncludeFile</name>
 <name lang="en">IncludeFile</name>

 <function>void IncludeFile( string item ,  string filename )</function>

 <description lang="fr">
 La fonction IncludeFile() sert  remplacer une variable de votre fichier template
 par le contenu d'un fichier.
 </description>

 <description lang="en">
 IncludeFile() serves to parse a variable by your file content.
 </description>

 <example>
 <?php

 $TPLN->IncludeFile("my_variable","file.html");

 ?>
 </example>
</submenu>
</docgeny>
**/
function IncludeFile($item,$file)
{
  // verification pour un bloc
  $this->Parse($item,"{#include(\"$file\");}"); // on replace par la commande include
  $this->f["$this->f_no"]["cmd_items"][] = $file; // on ajoute  la liste des f_cmd

}


/**
<docgeny>
<submenu name="getblocinfile" parent="functions">
 <name lang="fr">GetBlocInFile</name>
 <name lang="en">GetBlocInFile</name>

 <function>string GetBlocInFile( string item ,  string filename )</function>

 <description lang="fr">
 La fonction GetBlocInFile() sert  retourner un bloc entier du fichier template.
 </description>

 <description lang="en">
 The GetBlocInFile() function serves to return a whole block of the template file.
 </description>

 <example>
 <?php

 $bloc = $TPLN->GetBlocInFile("my_bloc");

 ?>
 </example>

</submenu>
</docgeny>
**/

function GetBlocInFile($path)
{
   $this->_PathVerify($path);

   $all_bloc = explode(".",$path);
   $bloc_name =  $all_bloc[count($all_bloc)-1];

   $bloc = $this->_CaptureBloc($bloc_name,$this->f["$this->f_no"]["buffer"]);
   return $bloc;
}

/**
<docgeny>
<submenu name="write" parent="functions">
 <name lang="fr">Write</name>
 <name lang="en">Write</name>

 <function>void Write()</function>

 <description lang="fr">
 La fonction Write() sert  imprimer le fichier template pars.
 </description>

 <description lang="en">
 The Write() function serves to print the file template.
 </description>

 <example>
 <?php

 $TPLN->Write();

 ?>
 </example>

</submenu>
</docgeny>
**/

 function Write()
 {
   $this->_Out();
   echo $this->f["$this->f_no"]["buffer"];
   $this->_CreateCache(TPLN_CACHE_DIR.$this->f["$this->f_no"]["name"]);
 }


/**
<docgeny>
<submenu name="output" parent="functions">
 <name lang="fr">Output</name>
 <name lang="en">Output</name>

 <function>void Output()</function>

 <description lang="fr">
 La fonction Output() sert  retourner le contenu du fichier template pars.
 </description>

 <description lang="en">
 The Output() function serves to return the file template.
 </description>

 <example>
 <?php

 $file_parsed = $TPLN->Output();

 ?>
 </example>

</submenu>
</docgeny>
**/
 function Output()
 {
   $this->_Out();
   $this->_CreateCache(TPLN_CACHE_DIR.$this->f["$this->f_no"]["name"]);
   return $this->f["$this->f_no"]["buffer"];
 }

/**
<docgeny>
<submenu name="savetemplate" parent="functions">
 <name lang="fr">SaveTemplate</name>
 <name lang="en">SaveTemplate</name>

 <function>void SaveTemplate(string $path)</function>

 <description lang="fr">
 La fonction SaveTemplate() sert  sauvegarder le contenu du fichier template pars.
 </description>

 <description lang="en">
 The SaveTemplate() function serves to save the file template.
 </description>

 <example>
 <?php

 $TPLN->SaveTemplate("tpl/newfile.html");

 ?>
 </example>

 <warning lang="fr">
 Si les dossiers du chemin dfini n'existe pas ils seront automatiquement crs.
 </warning>

 <warning lang="en">
 If folders countained in file path not exist they will be automatically created.
 </warning>


</submenu>
</docgeny>
**/
function SaveTemplate($path)
{

  // ajout d'un fonction pour la cration de dossier
   $all_dir = explode("/",$path);

   if(count($all_dir) > 1)
    {
      for($i=0;$i<count($all_dir)-1;$i++)
       {
          if($i > 0){$act_dir .= "/";}
          $act_dir .= $all_dir[$i];

          if(!is_dir($act_dir))
          {

            if(!@mkdir($act_dir,0755))
             {
              $this->_Error(7.1,$act_dir);
              return;
             }

           clearstatcache();
          }
       }
    }

   $output = $this->Output();
   $fp = @fopen($path, "w"); // ouverture du fichier
   @fwrite($fp,$output);
   @fclose($fp);
   clearstatcache(); // efface la mmoire tampon
}



/*************** additionnal *****************************/
// pour les fonctions spciales
function _ApplySpecialFunction($replace="", $functions="")
{

 if(empty($functions)){return $replace;}

   // y a t'il une fonction spciale ?
    $functions = explode("|",$functions);

    foreach($functions as $function_name)
     {
       if($function_name != "B" && $function_name != "I" && $function_name != "U" && !empty($function_name))
        {
         $replace = $function_name("$replace");
        }
     }

    // Bold, Italic, Underline
    if(in_array("B",$functions)){$replace = "<b>$replace</b>";}
    if(in_array("I",$functions)){$replace = "<i>$replace</i>";}
    if(in_array("U",$functions)){$replace = "<u>$replace</u>";}


 return $replace;
}


/*********************************************************
                          Blocs
*******************************************************/
// remplace le fils dans le pre
function _ParseEncapsuledBlocs($bloc)
{
  // on atteint le dernier bloc
  // on conserve son pre (adresse) et son fils (nom, structure)
  // on supprime le dernier membre de bloc
 while(count($bloc) > 1)
  {
    $b_ref = &$this->f["$this->f_no"]["def_blocs"];

    for($i=0;$i<count($bloc);$i++)
    {
        $b_ref = &$b_ref[$bloc[$i]];

        if($i == count($bloc) - 2) // capture du dernier pre
        {
          $father_parsed = &$b_ref["parsed"][count($b_ref["parsed"])-1];
         /* echo "father_parsed $father_parsed";
          echo "<hr>";  */
        }

        if($i == count($bloc) - 1) // capture du dernier fils
        {
          $children_name = $bloc[$i];
          $children_all_parsed = $b_ref["parsed"];
          $children_parsed = NULL;

          if(count($children_all_parsed) > 1)
          {
             for($j=0;$j<count($children_all_parsed)-1;$j++)
               {
                 $children_parsed .= $children_all_parsed[$j];
               }
          }
          else
          {
             if(count($children_all_parsed) == 1)
                 $children_parsed = $children_all_parsed[0];
          }
        }


      $b_ref = &$b_ref["children"];
    }

  /* echo "children_parsed $children_parsed";
   echo "<hr>";*/

    // on remplace le fils dans le pre
    $father_parsed = $this->_ReplaceBloc($children_name,$children_parsed,$father_parsed);

    /*echo "new father $father_parsed";
    echo "<hr>";    */

    // on supprime le dernier bloc
    unset($children_parsed);
    $bloc = array_slice($bloc,0,count($bloc)-1);
  }
}






function _ParseSubBlocs($level, $bloc_arr)
{
 $b_ref = &$this->f["$this->f_no"]["def_blocs"];

 foreach($bloc_arr as $cur_bloc)
 {
     $b_ref = &$b_ref["$cur_bloc"]["children"];
 }

 if(!is_array($b_ref))
     return;

  $b_names = array_keys($b_ref); // contiens tous les noms de blocs

  foreach($b_names as $bloc)
  {

    if(count($b_ref[$bloc]["children"]) > 0)
    {
      $bloc_arr[] = $bloc;
      $this->_ParseSubBlocs($level+1, $bloc_arr);
    }
    else
    {
      // le level est il le mme ?
      /*if($level < count($bloc_arr)-1)
       {
         $bloc_arr[count($bloc_arr)-1] = $bloc;
       }
      else
       {
          $bloc_arr[] = $bloc;
       } */

       /*echo "level $level <br><br>";

       echo "old bloc_arr avant le redimesionnement en ajoutant $bloc<br>";
       var_dump($bloc_arr);
       echo "<hr>";
       */
       $bloc_arr = array_slice($bloc_arr,0,$level+1);
       $bloc_arr[] = $bloc;

       /*echo "apres bloc_arr <br>";
       var_dump($bloc_arr); */

       $this->_ParseEncapsuledBlocs($bloc_arr);

    }

  }

}




function _ParseBigFathers()
{
   // on prend les pres parsed que l'on remplace dans le buffer
   $b_ref = &$this->f["$this->f_no"]["def_blocs"];

  // on prend les noms du parent BIG
  $b_names = array_keys($b_ref);

  $b_parsed = ""; // initialisation

  foreach($b_names as $bloc)
  {
    if(count($b_ref[$bloc]["parsed"]) == 1 && array_key_exists("0",$b_ref[$bloc]["parsed"]))
    {
       $b_parsed = $b_ref[$bloc]["parsed"][0];
    }

    // on regarde si le bloc a t en loop
    if(count($b_ref[$bloc]["parsed"]) > 1)
    {
      // on ne prend pas la premire
      for($i=0;$i<count($b_ref["$bloc"]["parsed"])-1;$i++)
          {
             $b_parsed .= $b_ref["$bloc"]["parsed"][$i];
           }
    }


    $this->f["$this->f_no"]["buffer"] = $this->_ReplaceBloc($bloc,$b_parsed,$this->f["$this->f_no"]["buffer"]);
    //unset($b_parsed);
    $b_parsed = "";
  }
}




function _GetBigBlocs()
{
  // touche le bloc
  $b_ref = &$this->f["$this->f_no"]["def_blocs"];
  $b_names = array_keys($b_ref);
  return $b_names;

}






function _Out()
{

  if(count($this->f["$this->f_no"]["def_blocs"]) > 0)
  {
     $b_ref = &$this->f["$this->f_no"]["def_blocs"];

     // on obtiens les noms des big blocs
     $big_blocs = $this->_GetBigBlocs();

     foreach($big_blocs as $bloc)
     {
       if(count($b_ref["$bloc"]["children"]) > 0)
        {
          $this->_ParseSubBlocs(0,(array)$bloc);
        }
     }

   $this->_ParseBigFathers(); // va compresser les bloc big et les parse au sein du buffer
  }

   $this->_ParseAllIncludeCmd();
   $this->_ParseChrono();
   $this->_ParseLogo();
   $this->_ParseVersion();
}

/********************************************
              Capture
*********************************************/
// fonction capture item
// et titre que ce d'un bloc
function _CaptureItems($subject="",$type="")
 {

   if(empty($subject))
    {
        $subject = $this->f["$this->f_no"]["buffer"]; // ds le fichier
        $blocs = $this->f["$this->f_no"]["shortcut_blocs"]["all"];
    }
   else
    {
     $subject = $this->_CaptureBloc($subject, $this->f["$this->f_no"]["buffer"]);

     // on capture les blocs du sujet
     $blocs = $this->_CaptureAllBlocs($subject);

    // on les efface
    foreach($blocs as $bloc)
      {
       $subject = $this->_ReplaceBloc($bloc,"",$subject);
      }

    }




    // exclision des espaces etc
  if($type == "PHP")
   {
     $motif = "/\{\\$([^ ;\*\$\\\,\\n\\t]+)?\}/msU";
   }
  else
   {
      $motif = "/\{([^ ;\.\*\$\\\,\\n\\t]+)?\}/msU";
   }

  $match = preg_match_all($motif,$subject,$tab);

   // on ddoublonne le tableau
   $one = array();

   if(count($tab[1]) > 0)
    {

     $one = array_unique($tab[1]);
    }

   return $one;
 }

// fonction qui capture un bloc sans ses motif
function _CaptureBloc($name,$subject)
 {

  if(empty($name))
    {
     $this->_Error(5,$this->f["$this->f_no"]["name"]);
     return;
    }

   $motif = "<bloc::$name>(.*)?<\\/bloc::$name>";
   $match = preg_match("/$motif/msU",$subject,$bloc);

   // error
   if(!$match)
    {
     $this->_Error(2,$this->f["$this->f_no"]["name"],$name);
     return;
    }

  $bloc = trim($bloc[1])."\n";

  return $bloc;
}

// fonction qui capture tous les blocs du sujet
function _CaptureAllBlocs($subject="")
 {
   if(empty($subject))
     {
       $subject = &$this->f["$this->f_no"]["buffer"];
     }

   $motif = "<bloc::([^ ;\.\*\$\\\,\\n\\t]+)?>";
   $match = preg_match_all("/$motif/U",$subject,$blocs);

   return $blocs[1];
}

// capture les items de tous les blocs du fichiers
function _CaptureItemsInEachBloc()
{
  if(count($this->f["$this->f_no"]["shortcut_blocs"]["all"]) == 0) {return;}

 foreach($this->f["$this->f_no"]["shortcut_blocs"]["all"] as $bloc)
  {
    $this->f["$this->f_no"]["shortcut_blocs"][$bloc]["items"] = $this->_CaptureItems($bloc);
  }

}

// preds les blocs du sujet
function _GetAllBlocs($subject)
{
   $motif = "<bloc::([^ ;\.\*\$\\\,\\n\\t]+)?>";
   preg_match_all("/$motif/U",$subject,$blocs);
   return $blocs[1];
}


// capture les instructions include au sein d'un template
function _CaptureIncludeCmd()
 {
   $motif = "\{#include\(([^ ;\*,\\n\\t]+)?\);\}";
   $match = preg_match_all("/$motif/U",$this->f["$this->f_no"]["buffer"],$tab);

   if(count($tab[1]) == 0)
    {
     return;
    }

   $one = array_unique($tab[1]);


   $this->f["$this->f_no"]["cmd_items"] = $one;

 }

/********************************************
              Verification
*********************************************/
/**
<docgeny>
<submenu name="itemexists" parent="functions">
 <name lang="fr">ItemExists</name>
 <name lang="en">ItemExists</name>

 <function>bool ItemExists(string $variable, [string $blocname])</function>

 <description lang="fr">
 La fonction ItemExists() sert savoir si une variable dans votre fichier template existe.
 </description>

 <description lang="en">
 ItemExists() function serves to know if a variable exists in the template.
 </description>

</submenu>
</docgeny>
**/

function ItemExists($item_name,$bloc="")
{
  if(empty($bloc))
  {

   if(@in_array($item_name,$this->f["$this->f_no"]["items"]))
   {
     return true;
   }
   else
    {
      return false;
    }
  }
  else
  {
    if(@in_array($item_name,$this->f["$this->f_no"]["shortcut_blocs"][$bloc]["items"]))
   {
     return true;
   }
   else
    {
      return false;
    }
  }
}

function _ItemVerify($item,$bloc="")
{

  if(empty($bloc))
  {
  if(!$this->ItemExists($item))
    {
     $this->_Error(1,$this->f["$this->f_no"]["name"],"",$item);
     return;
    }
  }
  else
  {
  if(!$this->ItemExists($item,$bloc))
    {
     $this->_Error(1.1,$this->f["$this->f_no"]["name"],$bloc,$item);
     return;
    }

  }

}

// verify que tous les blocs d'un chemin existe
function _PathVerify($path)
{
 $tab = @explode('.',$path);
 // on atteint le bloc et on lui ajoute les items
 for($i=0;$i<count($tab);$i++)
 {
     if(!$this->BlocExists($tab[$i]))
     {
       $this->_Error(2,$this->f["$this->f_no"]["name"],$tab[$i]);
       return;
     }
 }

}


/**
<docgeny>
<submenu name="blocexists" parent="functions">
 <name lang="fr">BlocExists</name>
 <name lang="en">BlocExists</name>

 <function>bool BlocExists(string blocname)</function>

 <description lang="fr">
 La fonction BlocExists() sert savoir si un bloc dans votre fichier template existe.
 </description>

 <description lang="en">
 BlocExists() function serves to know if a bloc exists in the template.
 </description>

</submenu>
</docgeny>
**/

function BlocExists($bloc_name)
{
   if(@in_array($bloc_name,$this->f["$this->f_no"]["shortcut_blocs"]["all"]))
   {
     return true;
   }
   else
    {
      return false;
    }
}


// verifie l'existence de la fin du bloc
function _EndBlocVerify()
{

  foreach($this->f["$this->f_no"]["shortcut_blocs"]["all"] as $bloc_name)
  {
   $motif = "/<\\/bloc::$bloc_name>/U";
   if(!preg_match($motif,$this->f["$this->f_no"]["buffer"]))
    {
     $this->_Error(8,$this->f["$this->f_no"]["name"],$bloc_name);
     return;
    }
  }
}


// verifie qu'il n'existe pas deux bloc similaires
function _DualBlocVerify()
{
  if(count($this->f["$this->f_no"]["shortcut_blocs"]["all"]) == 0) {return;}

  $blocs = array();


  foreach($this->f["$this->f_no"]["shortcut_blocs"]["all"] as $bloc){
      if(!in_array($bloc,$blocs)){
          $blocs[] = $bloc;}
      else{
          $this->_Error(2.1,$this->f["$this->f_no"]["name"],$bloc);}
  }

}


/********************************************
                    Parsing
*********************************************/
// item replace
function _ReplaceItem($name,$replace,$subject)
 {

   if(empty($name))
    {
    $this->_Error(6,$this->f["$this->f_no"]["name"]);
     return;
    }

   $str = preg_replace("/\{$name\}/U",$replace,$subject);
   return $str;
}

// fonction qui replace un bloc sans ses motifs
function _ReplaceBloc($name,$replace,$subject)
 {

  if(empty($name))
   {
    $this->_Error(5,$this->f["$this->f_no"]["name"]);
    return;
   }

   // c'est pas un tableau
   if(!is_array($name) && !is_array($replace))
    {
     $motif = "<bloc::$name>(.*)?<\\/bloc::$name>";
     $str = @preg_replace("/$motif/msU",$replace,$subject);
    }
    else
    {
     $str = @preg_replace($name,$replace,$subject);
    }
   return $str;
}

/**
<docgeny>
<submenu name="include" parent="functions">
 <name lang="fr">La Commande #include</name>
 <name lang="en">#include Command</name>

 <description lang="fr">
 la commande include est une variable qui se situe au sein de votre fichier template comme ceci
 {#include("my_file");}
 </description>

 <description lang="en">
 Include is a variable in your template like this  <b>{#include("my_file");}</b>
 </description>

 <example>
 <html>
 <head>{#include("header.html");}</head>
 <body>
 </body>
 </html>
 </example>

</submenu>
</docgeny>
**/
function _ParseIncludeCmd($file)
{
  // parsing du nom de fichier
  $file = str_replace(chr(34)," ",$file);
  $file = str_replace(chr(39)," ",$file);
  $file = trim($file);

  $filebuffer = $this->GetFile($file);

  if($this->_IsPhpFile($file))
  {
     $filebuffer = $this->_EvalHtml($filebuffer);
  }

  $file = str_replace("/","\/",$file);
  $motif = "/\{\#include\(\"$file\"\)\;\}|\{\#include\(\'$file\'\)\;\}/";
  $this->f["$this->f_no"]["buffer"] = preg_replace($motif,$filebuffer,$this->f["$this->f_no"]["buffer"]);
}



function _ParseAllIncludeCmd()
{
  if(count($this->f["$this->f_no"]["cmd_items"]) == 0){return;}

     foreach($this->f["$this->f_no"]["cmd_items"] as $filename)
      {
          $this->_ParseIncludeCmd($filename);
      }
}




function _EvalHtml($string)
{

 //$string = implode('', $string);

 ob_start();
 eval('?>' . $string);
 $string = ob_get_contents();
 ob_end_clean();
 return $string;

}



// est ce un fichier php ?
function _IsPhpFile($filename)
{
  $php_file_extension = "php|phtml|php4|php3"; // extensions de fichiers php

  if(preg_match("/\.($php_file_extension)$/i",basename($filename)))
   {
      return true;
   }
  else
   {
      return false;
   }
}



/****************************************************
                        Bloc
*****************************************************/

function _IsBlocDesired($path)
{
  if(preg_match("/[.]/",$path))
   {
    return true;
   }
  else
   {
    return false;
    }
}

// on vrifie que dans tous les blocs sont dfinis
function _IsDefined($path,$type="")
{
  if(empty($type))
   {
    $fathers_path = $this->_GetFathers($path); // rcupre les pres
   }

  if($type == 'NOITEM')
   {
    $fathers_path = $path; // rcupre les pres
   }


  if(@in_array($fathers_path, $this->f["$this->f_no"]["shortcut_blocs"]["used"],TRUE))
    {
      return true;
    }
  else
    {
      return false;
    }
}

// recupere le texte du bloc


// recupre l'item d'un path
function _GetItem($path)
{
  //if(!preg_match("/[.]/",$path)) {return;}

  $path_arr = explode(".", $path);
  return $path_arr[count($path_arr)-1];
}

// recupre le pre donc avant dernier bloc
function _GetFather($path)
{
  //if(!preg_match("/[.]/",$path)) {return;}

  $path_arr = explode(".", $path);
  return $path_arr[count($path_arr)-2];
}


// recupere les pres
function _GetFathers($path, $type="",$with_item=1)
{
  // if(!preg_match("/[.]/",$path)) {return;}
  $path_arr = explode(".", $path);

  if($with_item == 1)
   {
     $fathers_arr = array_slice($path_arr,0,count($path_arr)-1); // tous les noms sauf le dernier
   }
  else
   {
      $fathers_arr = $path_arr;
   }


  if($type == "ARRAY")
   {
     return $fathers_arr;
   }
   else
   {
    $fathers_path = join(".",$fathers_arr);
    return $fathers_path;
   }
}


function _DefineBloc($path)
{
  $fathers_path = $this->_GetFathers($path); // pas d'item
  $fathers_arr = $this->_GetFathers($path,'ARRAY'); // array

  foreach($fathers_arr as $bloc){
    $cur_arr[] = $bloc;

      if(count($cur_arr) == 1){
          $cur_path = $cur_arr[0];}
      else{
          $cur_path = join(".",$cur_arr);}

      // definition de tous les pres
      if(!$this->_IsDefined($cur_path,'NOITEM')){

          $this->_SavePath($cur_path);
          $a_bloc = $this->_InitialiazeBloc($bloc);

          // stockage
          $b_ref = &$this->f["$this->f_no"]["def_blocs"];

          $i = 0;
          foreach($cur_arr as $cur_bloc){
               $b_ref = &$b_ref["$cur_bloc"];

               if($i < (count($cur_arr)-1)){
                   $b_ref = &$b_ref["children"];}

               $i++;}

       $b_ref = $a_bloc;}
   }
}


function _InitialiazeBloc($bloc)
{
  $cur["structure"] = $this->_CaptureBloc($bloc,$this->f["$this->f_no"]["buffer"]);// structure
  $cur["parsed"][] = $cur["structure"];// parsed
  $cur["is_looped"] = 0;
  $cur["children"] = array();


  return $cur;
}


function _SavePath($fathers_path)
{
 $this->f["$this->f_no"]["shortcut_blocs"]["used"][] = $fathers_path; // on l'enregistre dans les blocs dfinis
}















/***************** Plug in ***********************************/
/*****************************************************
                      version
*****************************************************/
/**
<docgeny>
<submenu name="property" parent="functions">
 <name lang="fr">Variable Auto parse</name>
 <name lang="en">Automatic parsed variable</name>

 <description lang="fr">
 Si une de ces variables est prsentes dans votre fichier template,
 elles est automatiquement parse.
 <br>
 <ul>
   <li><b>{_Version}</b> est remplace par la version de TPLN</li>
   <li><b>{_Logo}</b> est remplace par le logo de TPLN</li>
   <li><b>{_Chrono}</b> est remplace par le temps de gnration du template.</li>
 </ul>
 </description>

 <description lang="en">
 If one of these variables is present in your template,
 they are automatically parsed.
 <br>
 <ul>
   <li><b>_Version</b> is replaced by the version of TPLN </ li>
   <li><b>_Logo</b> is replaced by the logo of TPLN </ li>
   <li><b>_Chrono</b> is replaced by the time of generation of the current template </ li>
 </ ul>
 </description>
</submenu>
</docgeny>
**/
function _ParseVersion()
{
    if($this->ItemExists("_Version")) // on place le logo
    {
     $this->Parse("_Version",$this->TPLN_version);
    }
}




/*****************************************************
                      logo
*****************************************************/
function _ParseLogo()
{
    if($this->ItemExists("_Logo")) // on place le logo
    {
     $this->Parse("_Logo","<a href=\"http://tpln.sourceforge.net\"><img src=\"http://tpln.sourceforge.net/logo.gif\" border=\"0\" width=\"83\" height=\"33\" alt=\"Visit HomePage\"></a>");
    }
}


/*****************************************************
                      chrono
*****************************************************/
/**
<docgeny>
<submenu name="setchrono" parent="functions">
 <name lang="fr">SetChrono</name>
 <name lang="en">SetChrono</name>

 <function>void SetChrono( string $type )</function>

 <description lang="fr">
 La fonction SetChrono(ALL) permet de pouvoir connaitre le temps de gnration total de votre template.
 Vous devez imprativement placer dans votre fichier template la variable <b>{_Chrono}</b>.
 </description>

 <description lang="en">
 The SetChrono(ALL) function permits to obtain the total chrono for all session.
 It can be with a variable in your template <b>{_Chrono}</b>.
 <br>
 If the variable <b>{_Chrono}</b> exists in your template, it will be automatically parsed by the timme execution of this session.
 </description>

 <warning lang="fr">
 Cette fonction doit tre place avant <a href="write.html">Write()</a> ou <a href="output.html">Output()</a>.
 </warning>

 <warning lang="en">
 This function must be placed before <a href="write.html">Write()</a> or <a href="output.html">Output()</a>.
 </warning>

</submenu>
</docgeny>
**/

function SetChrono($type="this")
 {
  $this->chrono_type["$this->f_no"] = $type;
 }

function _ParseChrono()
{
    if($this->ItemExists("_Chrono")) // on place le chrono
    {
     $this->_GetExecutionTime();
     $this->Parse("_Chrono",$this->f["$this->f_no"]["execution_time"]);
    }
}

// gestion du chrono
function _GetExecutionTime()
{
 $time = explode(' ',microtime());
 $fin = $time[1] + $time[0];

 // on veut les perf de cette session ou tous
 if($this->chrono_type["$this->f_no"] == "ALL")
  {
   $this->f["$this->f_no"]["execution_time"] = intval(10000 * ((double)$fin - (double)TPLN_CHRONO_STARTED)) / 10000;
   }
 else
  {
   $this->f["$this->f_no"]["execution_time"] = intval(10000 * ((double)$fin - (double)$this->f["$this->f_no"]["chrono_started"])) / 10000;
   }
}

/*****************************************************
                      Cache
*****************************************************/

// retourne la valeur de la premire ligne
function _InCachePeriod()
{
  $expire = $this->_GetTime(TPLN_CACHE_DIR.$this->f["$this->f_no"]["name"]);

  if($expire >= $this->f["$this->f_no"]["time_started"])
   {
    return true;
   }
  else
   {
    @unlink(TPLN_CACHE_DIR.$this->f["$this->f_no"]["name"]); // on efface le fichier
    return false;
   }
}

// prend le temps au dessus du fichier
function _GetTime()
{
 $fp = @fopen(TPLN_CACHE_DIR.$this->f["$this->f_no"]["name"], "r");// ouvre le fichier
 $expire = trim(@fgets($fp,12));
 //$data = fread($fp, filesize($file));
 @fclose($fp);

 clearstatcache();

 return $expire;
}

function _CreateCache()
{
 if(!$this->f["$this->f_no"]["create_cached_file"])
  {
   return;
  }
 else
  {
   $cache_file_content = $this->f["$this->f_no"]["cache_expire"]."\r".$this->f["$this->f_no"]["buffer"];

   // ajout d'un fonction pour la cration de dossier
   $all_dir = explode("/",$this->f["$this->f_no"]["name"]);
   if(count($all_dir) > 0)
    {
      $count = count($all_dir)-1; // on veut le dernier
      $act_dir = TPLN_CACHE_DIR;
      for($i=0;$i<$count;$i++)
       {
          $act_dir .= "/".$all_dir[$i];
          if(!is_dir($act_dir))
          {

            if(!@mkdir($act_dir,0755))
             {
              $this->_Error(7);
              return;
             }

           clearstatcache();
          }

       }
    }

   $fp = @fopen(TPLN_CACHE_DIR.$this->f["$this->f_no"]["name"], "w"); // ouverture du fichier
   $this->f["$this->f_no"]["buffer"] = @fwrite($fp,$cache_file_content);
   @fclose($fp);
   clearstatcache(); // efface la mmoire tampon
  }
}

function _GetCachedFile()
{
  $this->f["$this->f_no"]["create_cached_file"] = 0;

  $fp = @fopen(TPLN_CACHE_DIR.$this->f["$this->f_no"]["name"], "r");
  $tmp_expire = @fgets($fp,12); // pour la position du pointeur apres le temps
  $this->f["$this->f_no"]["buffer"] = @fread($fp,filesize(TPLN_CACHE_DIR.$this->f["$this->f_no"]["name"]));
  @fclose($fp);
  clearstatcache(); // efface la mmoire tampon
}

// verifie l'exitence du repertoire de cache et le crer ou cas ou
function _CacheDirExists()
{
 // le repertoire existe ?
 if(!is_dir(TPLN_CACHE_DIR))
 {
   if(!@mkdir(TPLN_CACHE_DIR,0755))
    {
      //$this->_AddTraceMsg();
      $this->_Error(7);
      return;
    }
   clearstatcache();
 }
}

}